import logging

from pyduofern.duofern_stick import DuofernStickThreaded

# from homeassistant.const import 'serial_port', 'config_file', 'code'
# found advice in the homeassistant creating components manual
# https://home-assistant.io/developers/creating_components/
# Import the device class from the component that you want to support
# from homeassistant.components import C

# Home Assistant depends on 3rd party packages for API specific code.
REQUIREMENTS = ['pyduofern==0.23.2']

_LOGGER = logging.getLogger(__name__)


def setup_platform(hass, config, add_devices, discovery_info=None):
    """Setup the Awesome Light platform."""

    # Assign configuration variables. The configuration check takes care they are
    # present.
    serial_port = config.get('serial_port')
    code = config.get('code')
    configfile = config.get('config_file')

    if 'duofern' not in hass.data:
        hass.data['duofern'] = {
            'stick': DuofernStickThreaded(serial_port=serial_port, system_code=code, config_file_json=configfile)}
        hass.data['duofern']['stick'].start()
        # Setup connection with devices/cloud
    stick = hass.data["duofern"]['stick']

    # Add devices
    add_devices(DuofernLight(device['id'], device['name'], stick) for device in stick.config['devices'] if
                device['id'].startswith('46'))


class DuofernLight(Light):
    def __init__(self, id, desc, stick):
        """Initialize the shutter."""
        self._id = id
        self._name = desc
        self._state = None
        self._brightness = None
        self._stick = stick

    @property
    def name(self):
        return self._name

    @property
    def is_on(self):
        try:
            _LOGGER.info(self._stick.duofern_parser.modules['by_code'][self._id])
            return self._stick.duofern_parser.modules['by_code'][self._id]['state'] == "on"
        except KeyError:
            return None

    def turn_on(self):
        self._stick.command(self._id, "on")
        # this is a hotfix because currently the state is not correctly detected from duofern
        self._stick.duofern_parser.modules['by_code'][self._id]['state'] = "on"

    def turn_off(self):
        self._stick.command(self._id, "off")
        # this is a hotfix because currently the state is not correctly detected from duofern
        self._stick.duofern_parser.modules['by_code'][self._id]['state'] = 0

    def update(self):
        pass



import logging

# from homeassistant.const import 'serial_port', 'config_file', 'code'
# found advice in the homeassistant creating components manual
# https://home-assistant.io/developers/creating_components/
# Import the device class from the component that you want to support
from homeassistant.components.cover import ATTR_POSITION, CoverDevice

from . import DuofernDevice
from .const import DOMAIN

_LOGGER = logging.getLogger(__name__)


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Setup the Awesome Light platform."""

    stick = hass.data[DOMAIN]['stick']

    # Add devices
    for cover in [dev for dev in stick.config['devices'] if not dev['id'].startswith('46')]:
        if cover['id'] not in hass.data[DOMAIN]['unique_ids']:
            hass.data[DOMAIN]['unique_ids'].add(cover['id'])
            add_entities([DuofernShutter(cover['id'], cover['name'], stick, hass)])


class DuofernShutter(DuofernDevice, CoverDevice):
    """Representation of Duofern cover type device."""

    def __init__(self, *args, **kwargs):
        """Initialize the shutter."""
        super().__init__(*args, **kwargs)

    @property
    def current_cover_position(self):
        """Return the display name of this cover."""
        try:
            return 100 - self._stick.duofern_parser.modules['by_code'][self._id]['position']
        except KeyError:
            return None

    @property
    def is_closed(self):
        """Return true if cover is close."""
        try:
            return self._stick.duofern_parser.modules['by_code'][self._id]['position'] == 100
        except KeyError:
            return False

    def open_cover(self):
        """roll up cover"""
        self._stick.command(self._id, "up")

    def close_cover(self):
        """close cover"""
        self._stick.command(self._id, "down")

    def stop_cover(self):
        """stop cover"""
        self._stick.command(self._id, "stop")

    def set_cover_position(self, **kwargs):
        """set position (100-position to make the default intuitive: 0%=closed, 100%=open"""
        position = kwargs.get(ATTR_POSITION)
        self._stick.command(self._id, "position", 100 - position)



import logging

# from homeassistant.const import 'serial_port', 'config_file', 'code'
import homeassistant.helpers.config_validation as cv
import voluptuous as vol
# found advice in the homeassistant creating components manual
# https://home-assistant.io/developers/creating_components/
# Import the device class from the component that you want to support
from homeassistant.components.light import Light, PLATFORM_SCHEMA

from . import DuofernDevice
from .const import DOMAIN

_LOGGER = logging.getLogger(__name__)

# Validation of the user's configuration
PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Optional('serial_port', default=None): cv.string,
    vol.Optional('config_file', default=None): cv.string,
    vol.Optional('code', default=None): cv.string,
})


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Setup the Awesome Light platform."""

    # Assign configuration variables. The configuration check takes care they are
    # present.

    stick = hass.data["duofern"]['stick']

    # Add devices
    for cover in [dev for dev in stick.config['devices'] if dev['id'].startswith('46')]:
        if cover['id'] not in hass.data[DOMAIN]['unique_ids']:
            hass.data[DOMAIN]['unique_ids'].add(cover['id'])
            add_entities([DuofernLight(cover['id'], cover['name'], stick, hass)])


class DuofernLight(DuofernDevice, Light):
    def __init__(self, *args, **kwargs):
        """Initialize the shutter."""
        super().__init__(*args, **kwargs)

    @property
    def is_on(self):
        try:
            _LOGGER.info(self._stick.duofern_parser.modules['by_code'][self._id])
            return self._stick.duofern_parser.modules['by_code'][self._id]['state'] == "on"
        except KeyError:
            return None

    def turn_on(self):
        self._stick.command(self._id, "on")
        # this is a hotfix because currently the state is not correctly detected from duofern
        self._stick.duofern_parser.modules['by_code'][self._id]['state'] = "on"

    def turn_off(self):
        self._stick.command(self._id, "off")
        # this is a hotfix because currently the state is not correctly detected from duofern
        self._stick.duofern_parser.modules['by_code'][self._id]['state'] = 0



import logging

# from homeassistant.const import 'serial_port', 'config_file', 'code'
# found advice in the homeassistant creating components manual
# https://home-assistant.io/developers/creating_components/
# Import the device class from the component that you want to support
from homeassistant.components.cover import ATTR_POSITION, CoverDevice
from homeassistant.core import callback

# Home Assistant depends on 3rd party packages for API specific code.
REQUIREMENTS = ['pyduofern']

from pyduofern.duofern_stick import DuofernStickThreaded

_LOGGER = logging.getLogger(__name__)


async def async_setup_platform(hass, config, add_devices, discovery_info=None):
    """Setup the Awesome Light platform."""

    # Assign configuration variables. The configuration check takes care they are
    # present.
    serial_port = config.get('serial_port')
    code = config.get('code')
    configfile = config.get('config_file')

    if 'duofern' not in hass.data:
        hass.data['duofern'] = {
            'stick': DuofernStickThreaded(serial_port=serial_port, system_code=code, config_file_json=configfile)}
        hass.data['duofern']['stick'].start()

    # Setup connection with devices/cloud
    stick = hass.data["duofern"]['stick']

    # Add devices
    add_devices(DuofernShutter(device['id'], device['name'], stick) for device in stick.config['devices'] if
                not device['id'].startswith('46'))


class DuofernShutter(CoverDevice):
    """Representation of Duofern cover type device."""

    def __init__(self, id, desc, stick):
        """Initialize the shutter."""
        self._id = id
        self._name = desc
        self._state = None
        self._brightness = None
        self._stick = stick
        self.opening = False
        self.closing = False
        self._stick.add_callback(self.update)

    @property
    def name(self):
        return self._name

    @property
    def current_cover_position(self):
        """Return the display name of this cover."""
        try:
            return 100 - self._stick.duofern_parser.modules['by_code'][self._id]['position']
        except KeyError:
            return None

    @property
    def is_closed(self):
        """Return true if cover is close."""
        try:
            return self._stick.duofern_parser.modules['by_code'][self._id]['position'] == 100
        except KeyError:
            return False

    @property
    def is_opening(self):
        return self.opening

    @property
    def is_closing(self):
        return self.closing

    async def async_open_cover(self):
        """roll up cover"""
        yield self._stick.command(self._id, "up")

    async def async_close_cover(self):
        """close cover"""
        yield self._stick.command(self._id, "down")

    async def async_stop_cover(self):
        """stop cover"""
        self._stick.command(self._id, "stop")

    async def async_set_cover_position(self, **kwargs):
        """set position (100-position to make the default intuitive: 0%=closed, 100%=open"""

        position = kwargs.get(ATTR_POSITION)
        if position > self.current_cover_position:
            self.opening = True
        elif position < self.current_cover_position:
            self.closing = True

        yield self._stick.command(self._id, "position", 100 - position)

    @callback
    def update(self):
        """Fetch new state data for this light.

        This is the only method that should fetch new data for Home Assistant.
        
        (no new data needs to be fetched, the stick updates itsself in a thread)
        (not the best style for homeassistant, I know. I'll port to asyncio if I find the time)
        """

        self.async_schedule_update_ha_state()


class TradfriLight(Light):
    """The platform class required by Home Assistant."""

    def __init__(self, light, api, gateway_id):
        """Initialize a Light."""
        self._api = api
        self._unique_id = "light-{}-{}".format(gateway_id, light.id)
        self._light = None
        self._light_control = None
        self._light_data = None
        self._name = None
        self._hs_color = None
        self._features = SUPPORTED_FEATURES
        self._available = True

        self._refresh(light)

    @property
    def unique_id(self):
        """Return unique ID for light."""
        return self._unique_id

    @property
    def min_mireds(self):
        """Return the coldest color_temp that this light supports."""
        return self._light_control.min_mireds

    @property
    def max_mireds(self):
        """Return the warmest color_temp that this light supports."""
        return self._light_control.max_mireds

    async def async_added_to_hass(self):
        """Start thread when added to hass."""
        self._async_start_observe()

    @property
    def available(self):
        """Return True if entity is available."""
        return self._available

    @property
    def should_poll(self):
        """No polling needed for tradfri light."""
        return False

    @property
    def supported_features(self):
        """Flag supported features."""
        return self._features

    @property
    def name(self):
        """Return the display name of this light."""
        return self._name

    @property
    def is_on(self):
        """Return true if light is on."""
        return self._light_data.state

    @property
    def brightness(self):
        """Return the brightness of the light."""
        return self._light_data.dimmer

    @property
    def color_temp(self):
        """Return the color temp value in mireds."""
        return self._light_data.color_temp

    @property
    def hs_color(self):
        """HS color of the light."""
        if self._light_control.can_set_color:
            hsbxy = self._light_data.hsb_xy_color
            hue = hsbxy[0] / (65535 / 360)
            sat = hsbxy[1] / (65279 / 100)
            if hue is not None and sat is not None:
                return hue, sat

    async def async_turn_off(self, **kwargs):
        """Instruct the light to turn off."""
        await self._api(self._light_control.set_state(False))

    async def async_turn_on(self, **kwargs):
        """Instruct the light to turn on."""
        params = {}
        transition_time = None
        if ATTR_TRANSITION in kwargs:
            transition_time = int(kwargs[ATTR_TRANSITION]) * 10

        brightness = kwargs.get(ATTR_BRIGHTNESS)

        if brightness is not None:
            if brightness > 254:
                brightness = 254
            elif brightness < 0:
                brightness = 0

        if ATTR_HS_COLOR in kwargs and self._light_control.can_set_color:
            params[ATTR_BRIGHTNESS] = brightness
            hue = int(kwargs[ATTR_HS_COLOR][0] * (65535 / 360))
            sat = int(kwargs[ATTR_HS_COLOR][1] * (65279 / 100))
            if brightness is None:
                params[ATTR_TRANSITION_TIME] = transition_time
            await self._api(
                self._light_control.set_hsb(hue, sat, **params))
            return

        if ATTR_COLOR_TEMP in kwargs and (self._light_control.can_set_temp or
                                          self._light_control.can_set_color):
            temp = kwargs[ATTR_COLOR_TEMP]
            if temp > self.max_mireds:
                temp = self.max_mireds
            elif temp < self.min_mireds:
                temp = self.min_mireds

            if brightness is None:
                params[ATTR_TRANSITION_TIME] = transition_time
            # White Spectrum bulb
            if (self._light_control.can_set_temp and
                    not self._light_control.can_set_color):
                await self._api(
                    self._light_control.set_color_temp(temp, **params))
            # Color bulb (CWS)
            # color_temp needs to be set with hue/saturation
            if self._light_control.can_set_color:
                params[ATTR_BRIGHTNESS] = brightness
                temp_k = color_util.color_temperature_mired_to_kelvin(temp)
                hs_color = color_util.color_temperature_to_hs(temp_k)
                hue = int(hs_color[0] * (65535 / 360))
                sat = int(hs_color[1] * (65279 / 100))
                await self._api(
                    self._light_control.set_hsb(hue, sat,
                                                **params))

        if brightness is not None:
            params[ATTR_TRANSITION_TIME] = transition_time
            await self._api(
                self._light_control.set_dimmer(brightness,
                                               **params))
        else:
            await self._api(
                self._light_control.set_state(True))

    @callback
    def _async_start_observe(self, exc=None):
        """Start observation of light."""
        # pylint: disable=import-error
        from pytradfri.error import PytradfriError
        if exc:
            _LOGGER.warning("Observation failed for %s", self._name,
                            exc_info=exc)

        try:
            cmd = self._light.observe(callback=self._observe_update,
                                      err_callback=self._async_start_observe,
                                      duration=0)
            self.hass.async_add_job(self._api(cmd))
        except PytradfriError as err:
            _LOGGER.warning("Observation failed, trying again", exc_info=err)
            self._async_start_observe()

    def _refresh(self, light):
        """Refresh the light data."""
        self._light = light

        # Caching of LightControl and light object
        self._available = light.reachable
        self._light_control = light.light_control
        self._light_data = light.light_control.lights[0]
        self._name = light.name
        self._features = SUPPORTED_FEATURES

        if light.light_control.can_set_color:
            self._features |= SUPPORT_COLOR
        if light.light_control.can_set_temp:
            self._features |= SUPPORT_COLOR_TEMP

    @callback
    def _observe_update(self, tradfri_device):
        """Receive new state data for this light."""
        self._refresh(tradfri_device)



import asyncio
import logging

# from homeassistant.const import 'serial_port', 'config_file', 'code'
import homeassistant.helpers.config_validation as cv
import voluptuous as vol
from homeassistant.helpers import discovery

from pyduofern.duofern_stick import DuofernStickAsync

DOMAIN = 'duofern'

# Home Assistant depends on 3rd party packages for API specific code.
REQUIREMENTS = ['pyduofern==0.23.5']

_LOGGER = logging.getLogger(__name__)

# Validation of the user's configuration
CONFIG_SCHEMA = vol.Schema({
    vol.Optional('serial_port', default=None): cv.string,
    vol.Optional('config_file', default=None): cv.string,
    vol.Optional('code', default=None): cv.string,
})


async def async_setup(hass, config):
    """Setup the duofern platform."""

    serial_port = config.get('serial_port')
    code = config.get('code')
    configfile = config.get('config_file')

    hass.data['duofern'] = {
        'stick': DuofernStickAsync(serial_port=serial_port, system_code=code, config_file_json=configfile)}
    hass.loop.create_task(hass.data['duofern']['stick'].handshake())

    # wait for handshake done (via future)
    await hass.data['duofern']['stick'].available

    # wait for a bit to allow duofern devices to call in
    await asyncio.sleep(10)

    hass.async_add_job(discovery.async_load_platform(hass, 'cover', DOMAIN, {}, config))
#    hass.async_add_job(discovery.async_load_platform(hass, 'sensor', DOMAIN, {}, config))



"""Config flow to configure Z-Wave."""
from collections import OrderedDict
import logging
import os
import stat

import voluptuous as vol

from homeassistant import config_entries

from .const import DOMAIN, CONF_CODE, CONF_SERIAL_PORT

_LOGGER = logging.getLogger(__name__)

@config_entries.HANDLERS.register(DOMAIN)
class DuofernFlowHandler(config_entries.ConfigFlow):
    """Handle a Z-Wave config flow."""

    VERSION = 1
    CONNECTION_CLASS = config_entries.CONN_CLASS_LOCAL_PUSH

    def __init__(self):
        """Initialize the Z-Wave config flow."""
        self.serial_port = CONF_SERIAL_PORT

    async def async_step_user(self, user_input=None):
        """Handle a flow start."""
        if self._async_current_entries():
            return self.async_abort(reason='one_instance_only')

        errors = {}

        fields = OrderedDict()
        fields[vol.Required(CONF_SERIAL_PORT,
                            default="/dev/ttyUSB0")] = str
        fields[vol.Required(CONF_CODE)] = str

        if user_input is not None:
            try:
                int(user_input[CONF_CODE], 16)
                conv_error = False
            except ValueError:
                conv_error = True
            if not stat.S_ISCHR(os.stat(user_input[CONF_SERIAL_PORT]).st_mode) or \
                    len(user_input[CONF_CODE]) != 4 or conv_error:
                errors['base'] = 'option_error'
                return self.async_show_form(
                    step_id='user',
                    data_schema=vol.Schema(fields),
                    errors=errors
                )

            return self.async_create_entry(
                title='duofern',
                data={
                    CONF_SERIAL_PORT: user_input[CONF_SERIAL_PORT],
                    CONF_CODE: user_input[CONF_CODE],
                },
            )

        return self.async_show_form(
          step_id='user', data_schema=vol.Schema(fields)
        )

#    async def async_step_import(self, info):
#        """Import existing configuration from Z-Wave."""
#        if self._async_current_entries():
#            return self.async_abort(reason='already_setup')

#        return self.async_create_entry(
#            title="Z-Wave (import from configuration.yaml)",
#            data={
#                CONF_SERIAL_PORT: info.get(CONF_SERIAL_PORT),
#                CONF_CODE: info.get(CONF_CODE),
#            },
#        )



import logging

# from homeassistant.const import 'serial_port', 'config_file', 'code'
import homeassistant.helpers.config_validation as cv
import voluptuous as vol
from homeassistant.helpers import discovery
from homeassistant.helpers.entity import Entity

# found advice in the homeassistant creating components manual
# https://home-assistant.io/developers/creating_components/
# Import the device class from the component that you want to support

# Home Assistant depends on 3rd party packages for API specific code.
REQUIREMENTS = ['pyduofern==0.24']

_LOGGER = logging.getLogger(__name__)

from .const import DOMAIN, DUOFERN_COMPONENTS, CONF_CODE, CONF_SERIAL_PORT

# from . import config_flow

PAIRING_SCHEMA = vol.Schema({
    vol.Optional('timeout', default=30): cv.positive_int,
})


def setup(hass, config):
    """Setup the Awesome Light platform."""

    # Assign configuration variables. The configuration check takes care they are
    # present.

    from pyduofern.duofern_stick import DuofernStickThreaded

    if config.get(DOMAIN) is not None:
        serial_port = config[DOMAIN].get(CONF_SERIAL_PORT)
        if serial_port is None:
            serial_port = "/dev/serial/by-id/usb-Rademacher_DuoFern_USB-Stick_WR04ZFP4-if00-port0"
        code = config[DOMAIN].get(CONF_CODE)
        if code is None:
            code = "affe"
    else:
        raise Exception("duofern needs configuration")

    def refresh():
        hass.data[DOMAIN]['stick'].sync_devices()
        for _component in DUOFERN_COMPONENTS:
            discovery.load_platform(hass, _component, DOMAIN, {}, config)

    hass.data['duofern'] = {
        'stick': DuofernStickThreaded(serial_port=serial_port, system_code=code,
                                      ephemeral=True, changes_callback=refresh),
        'unique_ids': set()}
    hass.data['duofern']['stick'].start()

    # Setup connection with devices/cloud
    stick = hass.data["duofern"]['stick']

    def start_pairing(call):
        hass.data[DOMAIN]['stick'].pair(call)

    hass.services.register(DOMAIN, 'start_pairing', start_pairing, PAIRING_SCHEMA)

    def sync_devices(call):
        refresh()

    hass.services.register(DOMAIN, 'sync_devices', sync_devices)

    refresh()

    return True


class DuofernDevice(Entity):
    """Representation of Duofern cover type device."""

    def __init__(self, id, desc, stick, hass):
        """Initialize the shutter."""
        self._id = id
        self._name = desc
        self._stick = stick
        hass.data[DOMAIN]['unique_ids'].add(id)

    @property
    def name(self):
        return self._name

    @property
    def device_info(self):
        return {
            'identifiers': {
                # Serial numbers are unique identifiers within a specific domain
                (DOMAIN, self._id)
            },
            'name': self.name,
            'manufacturer': "Rademacher",
            'model': "unknown",
            'sw_version': "unknown",
            # 'via_hub': (hue.DOMAIN, self.api.bridgeid),
        }

    @property
    def unique_id(self):
        return self._id

    def update(self):
        """Fetch new state data for this light.

        This is the only method that should fetch new data for Home Assistant.

        (no new data needs to be fetched, the stick updates itsself in a thread)
        (not the best style for homeassistant, I know. I'll port to asyncio if I find the time)
        """
        pass
        # self._state = True
        # self._brightness = None
